#!/usr/bin/env python
# -*- coding: utf-8 -*-
# encoding: utf-8
'''
messages.py

Processes messages utility library

Created by SWS on 2012-09-11.
Copyright (c) 2012 StoneWorkSolutions. All rights reserved.
'''
import json
import logging
import re
import redis
import traceback
import inspect

try:
	from stoneauth.middlewares import StoneThreadLocal as ThreadLocal
except:
	from django_tools.middlewares import ThreadLocal

logger = logging.getLogger('sws-tags')

level_translations = {
	'success': 'info',
	'broadcast': 'info',
}


def getTraceBackInfo():
	info = inspect.stack()[2][3]
	return info


def processErrorsForm(form):

	message = ''
	messages = []

	for field in form:

		if field.errors:
			message = re.sub('</(.*)>', '', unicode(str(field.errors), 'utf-8'))
			message = re.sub('<(.*)>', '', message)
			message = field.html_name + ': ' + message
			messages.append(message)

	return messages


def processMessages(message, typeMessage, timeOut=10000, rsyslog=True):

	msg = {}

	if rsyslog:
		try:
			called_from_view = getTraceBackInfo()
			if type(message) is list:
				for m in message:
					getattr(logger, level_translations.get(typeMessage, typeMessage))(' view: ' + called_from_view + ', user: ' + str(ThreadLocal.get_current_user()) + ', message: ' + m)
			elif type(message) is str:
				getattr(logger, level_translations.get(typeMessage, typeMessage))(' view: ' + called_from_view + ', user: ' + str(ThreadLocal.get_current_user()) + ', message: ' + message)
		except:
			pass

	msg[typeMessage] = (message, timeOut)

	return msg

# Send message to the specific channel
# Parametres:
# 	name channel (session_id or settings.CHANNEL_GLOBAL_MESSAGES)
# 	text message
# 	level (info, warning,error or debug)
# 	time visible


def sendMessages(channel, connection_redis, message='No messages', level='broadcast', time=5000, rsyslog=True):

	message = {"message": message, "level": level, "time": time}
	connection_redis.publish(channel, json.dumps(['user_message', message]))

	if rsyslog:
		try:
			called_from_view = getTraceBackInfo()
			getattr(logger, level_translations.get(level, level))(' channel: ' + channel + ', user: ' + str(ThreadLocal.get_current_user()) + ', message: ' + message['message'])
		except:
			pass

	return True


def sendEvent(channel, connection_redis, event='', data='', rsyslog=True):

	# cr = redis.Redis(**settings.REDIS_SERVERS['std_redis'])

	connection_redis.publish(channel, json.dumps([event, data]))

	if rsyslog:
		try:
			logger.debug('New event\n-Channel: {0}\n-User: {1}\n-Event: {2}\n-Data: {3}'.format(channel, ThreadLocal.get_current_user(), event, data))
		except:
			pass

	return True


def sendProgressBarEvent(channel, connection_redis, view_name='', value='', message='', error=False):
	sendEvent(channel, connection_redis, 'progressbar', {'view_name': view_name, 'value': value, 'message': message, 'error': error})


def sendBubbleNotificationEvent(channel, connection_redis, view_name='', value='', mode='add'):
	sendEvent(channel, connection_redis, 'bubblenotification', {'view_name': view_name, 'value': value, 'mode': mode})


def sendBubbleNotificationBlinkEvent(channel, connection_redis, view_name=''):
	sendEvent(channel, connection_redis, 'bubblenotificationblink', {'view_name': view_name})


def sendDashboardEvent(channel, connection_redis, data=''):
	sendEvent(channel, connection_redis, 'dashboard', {'data': data})


def swslog(typeMessage, message, e, gettraceback=True):

	called_from_view = getTraceBackInfo()

	try:
		if gettraceback:
			getattr(logger, typeMessage)(' -- ' + called_from_view + ', User: ' + str(ThreadLocal.get_current_user()) + ', Message {0}: '.format(typeMessage) + message + ' Exception: {0}'.format(traceback.format_exc()))
		else:
			getattr(logger, typeMessage)(' -- ' + called_from_view + ', User: ' + str(ThreadLocal.get_current_user()) + ', Message {0}: '.format(typeMessage) + message)
	except:
		pass



class StoneMRL(object):
	'''
		Method Rate limiter.
		Args:
		    param1 (redis object): Redis connection
	'''
	def __init__(self, redis_con):
		self.redis_con = redis_con

	def callRL(self, key, subkey, ex_num, limit_time, now):
		'''
			Register the lua script if it did not. And return the result of execute it.
			Args:
				param1(string): key to limit.
				param2(string): this is the name and score of set.
				param3(int): the number of executions at the same time
				param4(int): windows size
				param4(int): epoch
			Returns:
				int: 0 or 1. if returns 1 the method has to wait, when returns 0 the methos continues.
		'''
		if not hasattr(self.redis_con, 'method_rate_limiter'):
			self.redis_con.method_rate_limiter = self.redis_con.register_script(self._getLua())

		return self.redis_con.method_rate_limiter(keys=[ex_num, limit_time, now], args=[key, subkey])

	def _getLua(self):
		'''
			Return the lua script.
		'''
		return '''
			-- create vars
			local key = ARGV[1]
			local subkey = ARGV[2]
			local limit = tonumber(KEYS[1])
			local limit_time = tonumber(KEYS[1])
			local now = tonumber(KEYS[1])
			local reject = 1
			local stat1 = -1
			local total = 0

			--maintenance
			redis.call('ZREMRANGEBYSCORE', key, now-limit_time, now)

			--check
			stat1 = redis.call('ZRANK', key, subkey)
			if stat1 then
				if stat1 < limit then
					reject = 0
				else
					reject = 1
				end
				return reject
			end

			--add
			total = redis.call('ZCARD', key)
			if total < limit then
				reject = 0
			else
				reject = 1
			end
			redis.call('ZADD', key, subkey, subkey)
			return reject
		'''



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import errno
import uuid
# from circus import zmq
import zmq
# from circus.exc import CallError
# from circus.py3compat import string_types
# from circus.util import DEFAULT_ENDPOINT_DEALER, get_connection
# from zmq.utils.jsonapi import jsonmod as json


# def make_message(command, **props):
# 	return {"command": command, "properties": props or {}}


# def cast_message(command, **props):
# 	return {"command": command, "msg_type": "cast", "properties": props or {}}


# def make_json(command, **props):
# 	return json.dumps(make_message(command, **props))


# class CircusClient(object):
	# def __init__(self, context=None, endpoint=DEFAULT_ENDPOINT_DEALER,
	#              timeout=5.0, ssh_server=None, ssh_keyfile=None):
	#     self.context = context or zmq.Context.instance()
	#     self.endpoint = endpoint
	#     self._id = uuid.uuid4().hex
	#     self.socket = self.context.socket(zmq.DEALER)
	#     self.socket.setsockopt(zmq.IDENTITY, self._id)
	#     self.socket.setsockopt(zmq.LINGER, 0)
	#     get_connection(self.socket, endpoint, ssh_server, ssh_keyfile)
	#     self.poller = zmq.Poller()
	#     self.poller.register(self.socket, zmq.POLLIN)
	#     self._timeout = timeout
	#     self.timeout = timeout * 1000

	# def stop(self):
	#     self.socket.close()

	# def send_message(self, command, **props):
	#     return self.call(make_message(command, **props))

	# def call(self, cmd):
	#     if not isinstance(cmd, string_types):
	#         try:
	#             cmd = json.dumps(cmd)
	#         except ValueError as e:
	#             raise CallError(str(e))

	#     try:
	#         self.socket.send(cmd)
	#     except zmq.ZMQError, e:
	#         raise CallError(str(e))

	#     while True:
	#         try:
	#             events = dict(self.poller.poll(self.timeout))
	#         except zmq.ZMQError as e:
	#             if e.errno == errno.EINTR:
	#                 continue
	#             else:
	#                 print str(e)
	#                 raise CallError(str(e))
	#         else:
	#             break

	#     if len(events) == 0:
	#         raise CallError("Timed out.")

	#     for socket in events:
	#         msg = socket.recv()
	#         try:
	#             return json.loads(msg)
	#         except ValueError as e:
	#             raise CallError(str(e))


def reload_all_workers(circus_endpoint, app_name):
	# c = CircusClient(endpoint=circus_endpoint)
	# cmd = '{"command": "signal", "properties": {"name": "{0}", "signum": 3}}'.format(app_name)
	# cmd = '{"command": "signal", "properties": {"name": "'+app_name+'", "signum": 3}}'
	# circus_response = c.call(cmd)
	# print "circus_response",circus_response
	# if circus_response['status'] == 'ok':
	# 	return 'OK,ALL_WORKERS_RELOADING,{0}'.format(circus_response['time'])
	# else:
	# 	return 'KO,RELOAD_ALL_WORKERS_FAILED'
	pass